﻿using DryIoc.ImTools;
using LiveCharts;
using LiveCharts.Wpf;
using Mysqlx.Crud;
using MySqlX.XDevAPI.Common;
using Newtonsoft.Json;
using Org.BouncyCastle.Utilities;
using StudentManager.Common;
using StudentManager.Data;
using StudentManager.Extensions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Timer = StudentManager.Common.Timer;

namespace StudentManager.Model
{
	class MenuBar : BindableBase
	{
		private string icon;
		/// <summary>
		/// Icon
		/// </summary>
		public string Icon
		{
			get { return icon; }
			set { icon = value; }
		}

		private string title;

		public string Title
		{
			get { return title; }
			set { title = value; }
		}


		private string nameSpace;

		public string NameSpace
		{
			get { return nameSpace; }
			set { nameSpace = value; }
		}


	}

	class Students : BindableBase
	{

		private ObservableCollection<StudentInfo> studentDatas = new ObservableCollection<StudentInfo>();
		public ObservableCollection<StudentInfo> StudentDatas { get { return studentDatas; } private set { studentDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<QuestionData> questionDatas = new ObservableCollection<QuestionData>();
		public ObservableCollection<QuestionData> QuestionDatas { get { return questionDatas; } private set { questionDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<QuestionData> homeworkTestData = new ObservableCollection<QuestionData>();
		public ObservableCollection<QuestionData> HomeworkTestData { get { return homeworkTestData; } private set { homeworkTestData = value; RaisePropertyChanged(); } }

		private ObservableCollection<QuestionData> preToRemove = new ObservableCollection<QuestionData>();
		private ObservableCollection<QuestionData> addQuestionDatas = new ObservableCollection<QuestionData>();
		public ObservableCollection<QuestionData> ADDQuestionDatas { get { return addQuestionDatas; } private set { addQuestionDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<DetailErrorInfo> errorSetDatas = new ObservableCollection<DetailErrorInfo>();
		public ObservableCollection<DetailErrorInfo> ErrorSetDatas { get { return errorSetDatas; } private set { errorSetDatas = value; RaisePropertyChanged(); } }

		private ObservableCollection<CursonQuestionsData> cQDatas = new ObservableCollection<CursonQuestionsData>();
		public ReadOnlyObservableCollection<CursonQuestionsData> CQDatas { get; private set; }


		private ObservableCollection<DetailHomeWorkSetInfo> homeWorkSet = new ObservableCollection<DetailHomeWorkSetInfo>();
		public ObservableCollection<DetailHomeWorkSetInfo> HomeWorkSet { get { return homeWorkSet; } private set { homeWorkSet = value; RaisePropertyChanged(); } }

		private ObservableCollection<DetailHomeWorkInfo> homeWork = new ObservableCollection<DetailHomeWorkInfo>();
		public ObservableCollection<DetailHomeWorkInfo> HomeWork { get { return homeWork; } private set { homeWork = value; RaisePropertyChanged(); } }

		private ObservableCollection<MenuBar> menuBars = new ObservableCollection<MenuBar>();
		public ReadOnlyObservableCollection<MenuBar> MenuBars { get; private set; }



		private int totalQuestions;
		public int TotalQuestions
		{
			get { return totalQuestions; }
			set { totalQuestions = value; }
		}


		private StudentInfo selectedStudent;
		public StudentInfo SelectedStudent
		{
			get { return selectedStudent; }
			set { selectedStudent = value; RaisePropertyChanged(); }
		}


		private DetailHomeWorkSetInfo selectedHomeWorkSet;
		public DetailHomeWorkSetInfo SelectedHomeWorkSet
		{
			get { return selectedHomeWorkSet; }
			set { selectedHomeWorkSet = value; RaisePropertyChanged(); }
		}

		private DetailHomeWorkInfo selectedHomeWork;
		public DetailHomeWorkInfo SelectedHomeWork
		{
			get { return selectedHomeWork; }
			set { selectedHomeWork = value; RaisePropertyChanged(); }
		}

		private StudentInfo studentInfo;

		public StudentInfo StudentInfo
		{
			get { return studentInfo; }
			set { studentInfo = value; RaisePropertyChanged(); }
		}

		private DetailErrorSetInfo errorSet;

		public DetailErrorSetInfo ErrorSet
		{
			get { return errorSet; }
			set { errorSet = value; RaisePropertyChanged(); }
		}

		private uint lesson;

		public uint Lesson
		{
			get { return lesson; }
			set { lesson = value; RaisePropertyChanged(); }
		}


		private bool isAddPublicQuestionsLib;

		public bool IsAddPublicQuestionsLib
		{
			get { return isAddPublicQuestionsLib; }
			set { isAddPublicQuestionsLib = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }

		}

		private bool isNeedErrorset;

		public bool IsNeedErrorset
		{
			get { return isNeedErrorset; }
			set { isNeedErrorset = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }
		}

		private bool isControlQuestionNum;

		public bool IsControlQuestionNum
		{
			get { return isControlQuestionNum; }
			set { isControlQuestionNum = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }
		}


		private uint publicLesson;

		public uint PublicLesson
		{
			get { return publicLesson; }
			set { publicLesson = value; OnPublicHomeworkSelectionChanged(); RaisePropertyChanged(); }
		}


		private int questionCount;

		public int QuestionCount
		{
			get { return questionCount; }
			set { questionCount = value; RaisePropertyChanged(); }
		}

		private QuestionData selectedQuestion = new QuestionData();

		public QuestionData SelectedQuestion
		{
			get { return selectedQuestion; }
			set { selectedQuestion = value; RaisePropertyChanged(); }
		}



		public SeriesCollection seriesCollection;
		public SeriesCollection SeriesCollection
		{
			get { return seriesCollection; }
			set { seriesCollection = value; RaisePropertyChanged(); }
		}

		public List<string> labels;
		public List<string> Labels
		{
			get { return labels; }
			set { labels = value; RaisePropertyChanged(); }
		}


		private string currentTime;

		public string CurrentTime
		{
			get { return currentTime; }
			set { currentTime = value; RaisePropertyChanged(); }
		}


		private readonly IRegionManager regionManager;
		private readonly IEventAggregator aggregator;

		public DelegateCommand<string> RegionTo { get; set; }
		public DelegateCommand<StudentInfo> SelectedCommand { get; set; }
		public DelegateCommand<DetailHomeWorkInfo> HomeWorkSelectedCommand { get; set; }
		public DelegateCommand<DetailHomeWorkSetInfo> HomeWorkSetSelectedCommand { get; set; }
		public DelegateCommand DeleteQuestionCommand { get; set; }
		public DelegateCommand UpdateQuestionCommand { get; set; }
		public DelegateCommand<MouseButtonEventArgs> ShowContextMenuCommand { get; set; }
		public DelegateCommand<QuestionData> RemoveNewColumnCommand { get; set; }
		public DelegateCommand<QuestionData> SelectedQuestionChangedCommand { get; set; }
		public DelegateCommand RegionToDetailCommand { get; set; }
		public DelegateCommand UpdateHomeWorkCommand { get; set; }
		public DelegateCommand DeleteHomeworkCommand { get; set; }
		public DelegateCommand AddNewColumnCommand { get; set; }
		public DelegateCommand SubmitAddQuestionsCommand { get; set; }
		public DelegateCommand ClearnAddQuestionsCommand { get; set; }
		public DelegateCommand SaveAllCommand { get; set; }
		public DelegateCommand PublicHomeWorkCommand { get; set; }
		public DelegateCommand FreshAllCommand { get; set; }

		Students(IRegionManager regionManager, IEventAggregator aggregator)
		{


			CreateMenuBar();
			CurrentTime = DateTime.Now.ToString("HH:mm");

			Timer timer = new Timer(() => {
				CurrentTime = DateTime.Now.ToString("HH:mm");
			});

			FileSystem.Instance.AutoSaveInvoke += () =>
			{
				aggregator.SendMessage(" 将在10秒后自动保存 ");
			};

			FileSystem.Instance.AutoSave();

			RegionTo = new DelegateCommand<string>((x) =>
			{
				regionManager.Regions[PrismManager.MainRegionName].RequestNavigate(x.ToString());
				if (x == "DetailCheckView")
				{
					RegionToDetailCheckView();
				}
				if (x == "DetailView")
				{
					RegionToDetailView();
				}
				RegionToStudents();
			});

			UpdateHomeWorkCommand = new DelegateCommand(() => UpdateHomeWork());

			SelectedCommand = new DelegateCommand<StudentInfo>((x) =>
			{
				if (x == null) return;
				SelectedStudent = x;

				UpdateGird();
				RegionToDetailView();
			});

			HomeWorkSetSelectedCommand = new DelegateCommand<DetailHomeWorkSetInfo>((x) =>
			{
				if (x == null || SelectedStudent == null) return;
				SelectedHomeWorkSet = x;
				HomeWork = SelectedStudent.HomeWorkSet.GetDetailHomeWorkList(SelectedHomeWorkSet.Lesson);
			});

			HomeWorkSelectedCommand = new DelegateCommand<DetailHomeWorkInfo>((e) =>
			{
				if (e == null) return;
				//selectedHomeWork = e;
				e.Status = true;
			});

			DeleteQuestionCommand = new DelegateCommand(() =>
			{
				if (SelectedQuestion == null)
				{
					aggregator.SendMessage($" 当前没有选中任何题目 ");
					return;
				}
				var c = MessageBox.Show("确定删除吗？", "确认", MessageBoxButton.OKCancel);
				if (c != MessageBoxResult.OK) return;
				if (!SQLHelper.Instance.Delete(SelectedQuestion)) return;
				QuestionLib.Instance.Remove(SelectedQuestion);
				QuestionDatas = QuestionLib.Instance.GetAllQuestion();
				aggregator.SendMessage($" 成功删除 ");

				FileSystem.Instance.SaveAll();

			});

			UpdateQuestionCommand = new DelegateCommand(() =>
			{
				if (SelectedQuestion == null) return;
				var c = MessageBox.Show("确定修改吗？", "确认", MessageBoxButton.OKCancel);
				if (c != MessageBoxResult.OK) return;
				QuestionLib.Instance.Update(SelectedQuestion);
				SelectedQuestion.UpdateTime = DateTime.Now;
				SQLHelper.Instance.UpdateQuestion(SelectedQuestion);

				aggregator.SendMessage("已更新问题信息");

				FileSystem.Instance.SaveAll();
			});

			SelectedQuestionChangedCommand = new DelegateCommand<QuestionData>((x) =>
			{
				if (x == null) return;
				SelectedQuestion = x;

			});

			ClearnAddQuestionsCommand = new DelegateCommand(() =>
			{
				var c = MessageBox.Show("确定清除吗？", "清除所有题目", MessageBoxButton.OKCancel);
				if (c != MessageBoxResult.OK) return;
				ADDQuestionDatas.Clear();
			});

			SaveAllCommand = new DelegateCommand(() => FileSystem.Instance.SaveAll());
			FreshAllCommand = new DelegateCommand(() =>
			{
				FileSystem.Instance.FreashAllInfo();
				Reload();
			});
			PublicHomeWorkCommand = new DelegateCommand(() =>
			{
				var result = HomeWorkManager.Instance.PublicHomework(PublicLesson, IsAddPublicQuestionsLib, isNeedErrorset);
				aggregator.SendMessage($"总发布{result.Total}个作业， {result.Correct}已发布, {result.Error}未发布");
			});


			DeleteHomeworkCommand = new DelegateCommand(() =>
			{

				var cq = new CursonQuestionsData { Lesson = SelectedHomeWorkSet.Lesson, UID = SelectedStudent.UID };

				if (!SQLHelper.Instance.Delete(cq))
				{
					aggregator.SendMessage(" 服务器删除失败 ");
					return;
				}

				if (SelectedHomeWorkSet != null && SelectedStudent != null)
					SelectedStudent.HomeWorkSet.DeleteHomework(SelectedHomeWorkSet.Lesson);
				else
				{
					aggregator.SendMessage(" 本地数据错误，已为你从服务器更新 ");
					QuestionLib.Instance.FreshAllQuestion();
				}

				if (SelectedStudent != null)
				{
					HomeWorkSet = SelectedStudent.HomeWorkSet.GetDetailHomeWorkSetList();
					SelectedStudent.HomeWorkSet.UpdateDate();
				}


				aggregator.SendMessage(" 删除成功 ");


				FileSystem.Instance.SaveAll();


			});


			SubmitAddQuestionsCommand = new DelegateCommand(() =>
			{

				Debug.Assert(Lesson != 0);

				preToRemove.Clear();

				foreach (var item in ADDQuestionDatas)
				{
					item.Id = ConvertToHashInt(item.Stem);
					item.Lesson = Lesson;
					if (!(QuestionLib.Instance.Get(item.Id) == null)) continue;
					if (string.IsNullOrEmpty(item.Stem) || string.IsNullOrEmpty(item.Source) || string.IsNullOrEmpty(item.Answer)) continue;
					if (!SQLHelper.Instance.Add(item))
					{
						aggregator.SendMessage($" {item.Stem}服务器添加失败, ID 可能重复 ");
						continue;
					}

					QuestionLib.Instance.Add(item);
					preToRemove.Add(item);
				}

				foreach (var item in preToRemove)
				{
					ADDQuestionDatas.Remove(item);
				}

				aggregator.SendMessage($" {preToRemove.Count}添加成功， {ADDQuestionDatas.Count}添加失败 ");

				QuestionLib.Instance.Save();
			});


			Load();

			StudentDatas = new ObservableCollection<StudentInfo>(studentDatas);
			QuestionDatas = new ObservableCollection<QuestionData>(questionDatas);
			ErrorSetDatas = new ObservableCollection<DetailErrorInfo>(errorSetDatas);
			MenuBars = new ReadOnlyObservableCollection<MenuBar>(menuBars);
			HomeWorkSet = new ObservableCollection<DetailHomeWorkSetInfo>(homeWorkSet);


			UpdateGird();

			this.regionManager = regionManager;
			this.aggregator = aggregator;
		}

		private void UpdateGird()
		{
			if (SelectedStudent == null) return;

			SelectedStudent.HomeWorkSet.UpdateDataView();

			SeriesCollection = new SeriesCollection
			{
				new LineSeries
				{
					Title = "错题数量",
					Values = SelectedStudent.HomeWorkSet.ErrorCounts,
				},
				new LineSeries
				{
					Title = "正确数量",
					Values = SelectedStudent.HomeWorkSet.CorrectCounts,
				}
			};

			Labels = SelectedStudent.HomeWorkSet.Lessons;
		}


		private int ConvertToHashInt(string item)
		{
			var hash = SHA256.Create();
			var result = hash.ComputeHash(Encoding.UTF8.GetBytes(item));
			return BitConverter.ToInt32(result, 0);
		}



		private void UpdateHomeWork()
		{
			// update error set and local homework

			bool isChanged = false;

			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Questions.ForEach(x =>
			{
				if (x.Status != HomeWork.FirstOrDefault(hw => hw.PID == x.ID).Status || selectedHomeWorkSet.Status == false)
				{
					isChanged = true;
					x.Status = HomeWork.FirstOrDefault(hw => hw.PID == x.ID).Status;
					SelectedStudent.ErrorSet.AddQuestion(x);
					ErrorBase error = SelectedStudent.ErrorSet.Get(x.ID) ?? new ErrorBase { QuestionBase = new QuestionBase { ID = x.ID, Status = x.Status } };
					if (!SQLHelper.Instance.UpdateErrorSet(SelectedStudent.UID, error))
					{
						aggregator.SendMessage($"{error.QuestionBase.ID} 更新错题集数据失败");
					}
				}
			});

			if (!isChanged)
			{
				aggregator.SendMessage($" 当前作业没有变更 ");
				return;
			}

			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).UpdateData();
			SelectedStudent.HomeWorkSet.UpdateDate();
			SQLHelper.Instance.UpdateErrorSetUserInfo(SelectedStudent.UID, (uint)SelectedStudent.ErrorSet.TotalCount, (uint)SelectedStudent.ErrorSet.CorrectCount);
			// update homework other info and online info
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Status = true;
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).DateTime = DateTime.Now;
			List<int> correctHomeworkArray = new List<int>();
			List<int> totalHomeworkArray = new List<int>();
			SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Questions.ForEach(x =>
			{
				totalHomeworkArray.Add(x.ID);
				if (x.Status != false) correctHomeworkArray.Add(x.ID);

			});
			SQLHelper.Instance.UpdateHomework(SelectedStudent.UID, SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Lesson,
				JsonConvert.SerializeObject(totalHomeworkArray),
				JsonConvert.SerializeObject(correctHomeworkArray), SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).DateTime,
				SelectedStudent.HomeWorkSet.Get(SelectedHomeWorkSet.Lesson).Status);


			SQLHelper.Instance.UpdateHomeworkTotalInfo(SelectedStudent.UID, (uint)SelectedStudent.HomeWorkSet.TotalQuestionCount, (uint)SelectedStudent.HomeWorkSet.TotalErrorQuestionCount);

			aggregator.SendMessage($" 更新错题集数据和家庭作业成功 ");
			selectedHomeWorkSet.Status = true;


			HomeWorkSet = SelectedStudent.HomeWorkSet.GetDetailHomeWorkSetList();
			FileSystem.Instance.SaveAll();
			//TODO: Notify
		}

		private void UpdateData()
		{

		}

		private void RegionToDetailView()
		{
			if (selectedStudent == null) return;
			ErrorSetDatas = selectedStudent.ErrorSet.GetDetailErrorQuestionList();
			HomeWorkSet = selectedStudent.HomeWorkSet.GetDetailHomeWorkSetList();
		}

		private void RegionToDetailCheckView()
		{
			if (selectedHomeWorkSet == null) return;
			HomeWork = SelectedStudent.HomeWorkSet.GetDetailHomeWorkList(SelectedHomeWorkSet.Lesson);
		}

		private void RegionToStudents()
		{

		}

		void CreateMenuBar()
		{
			menuBars.Add(new MenuBar() { Icon = "ForumPlusOutline", Title = "学生", NameSpace = "StudentsView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "详情", NameSpace = "DetailView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "题库", NameSpace = "QuestionsView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "审批", NameSpace = "CheckView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "发布家庭作业", NameSpace = "AddHomeWork" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "出题", NameSpace = "AddQuestionView" });
		}

		private void Load()
		{
			StudentLib.Load();
			QuestionLib.Instance.Load();
			Reload();
		}

		private void Reload()
		{
			studentDatas.Clear();
			questionDatas.Clear();
			StudentDatas = StudentLib.GetAllStudents();
			QuestionDatas = QuestionLib.Instance.GetAllQuestion();
		}


		private void OnPublicHomeworkSelectionChanged()
		{
			HomeworkTestData.Clear();
			if (IsAddPublicQuestionsLib)
			{
				HomeworkTestData = QuestionLib.Instance.GetAllQuestionByLesson(PublicLesson);
			}
			if (IsNeedErrorset)
			{
				if (SelectedStudent != null)
				{
					foreach (var item in SelectedStudent.ErrorSet.GetDetailErrorQuestionList())
					{
						homeworkTestData.Add(item.QuestionData);
					}
				}

			}
		}

	}
}
